package com.hive.sample

import android.app.Activity
import android.content.Intent
import android.view.View
import com.com2us.hivesdk.normal.freefull.google.global.android.common.R
import com.hive.AuthV4
import com.hive.ResultAPI
import com.hive.sample.uihelper.ViewPagerBase
import com.hive.talkplus.HiveTalkPlus
import com.hive.talkplus.HiveTalkPlusUser


/**
 * HIVE SDK 의 TalkPlus 기능 테스트 뷰
 *
 * @author hife
 */
class TalkPlusTestView(activity: Activity, view: View, title: String) : ViewPagerBase(activity, view, title), View.OnClickListener {


    init {
        initUi()
    }


    private fun initUi() {

        createLogView(R.id.logview, R.id.btnLogDialog, R.id.btnLogClear)

        this.view.findViewById<View>(R.id.btnHiveTalkPlusLogin).setOnClickListener(this)
    }


    override fun onClick(v: View) {

        when (v.id) {

            R.id.btnHiveTalkPlusLogin -> {
                addLog("\n\n=== 하이브 톡플러스 시작 ===\n")
                addLog("HiveTalkPlus.loginWithToken() Called\n")

                HiveTalkPlus.init { isInit, message ->
                    if (isInit) {
                        AuthV4.getHiveTalkPlusLoginToken(object : AuthV4.AuthV4GetHiveTalkPlusLoginTokenListener {
                            override fun onAuthV4GetHiveTalkPlusLoginToken(resultApi: ResultAPI, loginToken: String?) {

                                if (resultApi.isFailure) {
                                    showAlertDialog("HiveTalkPlus", "getHiveTalkPlusLoginToken failed: ($resultApi)")

                                } else {
                                    val playerInfo = AuthV4.getPlayerInfo()

                                    HiveTalkPlus.loginWithToken(
                                        playerInfo?.playerId.toString(),
                                        loginToken,
                                        playerInfo?.playerName,
                                        playerInfo?.playerImageUrl,
                                        null,
                                        object : HiveTalkPlus.CallbackListener<HiveTalkPlusUser?> {
                                            override fun onSuccess(t: HiveTalkPlusUser?) {
                                                addLog("HiveTalkPlus.loginWithToken() onSuccess: $t\n")

                                                val intent = Intent(activity, com.hive.sample.talkplus.MainActivity::class.java)
                                                activity.startActivity(intent)
                                            }

                                            override fun onFailure(i: Int, e: Exception?) {
                                                showAlertDialog("HiveTalkPlus", "HiveTalkPlus login failed" +
                                                        "\n  error code: $i" +
                                                        "\n  ${e.toString()}")
                                            }
                                        }
                                    )

                                }
                            }
                        })

                    } else {
                        showAlertDialog(
                            "HiveTalkPlus",
                            "HiveTalkPlus init failed." +
                                "\n    $message"
                        )
                    }
                }

            }

        }

    }
}
